;; linesegments.scm -*-scheme-*-
;;
;; Draw two lines AB and CD then divide each line into N
;; equally sized divisions. Draw a line from A to D, 
;; then continue drawing lines from corresponding points 
;; on the two lines. The end result is a pleasing curved
;; pattern as well as interesting interference patterns.

;; More info at http://members.optusnet.com.au/~charles57/GIMP
;; Version 1.0
;;
;; Copyright (C) 2008 by Charles Cave <charlesweb@optusnet.com.au>
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (script-fu-linesegments    img drw
                                   ndiv
                                   ax ay bx by cx cy dx dy
                                   lineset
                                   drawab drawcd
                                   brush
                                   color)
    (let* (
            (axinc 0)
            (ayinc 0)

            (cxinc 0)
            (cyinc 0)

            (x1 0)
            (x2 0)
            (y1 0)
            (y2 0)

            (niter 0)

            (*linepoints* (cons-array 4 'double))
          )

      (set! axinc (/ (- bx ax) ndiv) )
      (set! ayinc (/ (- by ay) ndiv) )

      (set! cxinc (/ (- dx cx) ndiv) )
      (set! cyinc (/ (- dy cy) ndiv) )

;; Prepare to draw
  (gimp-context-push)
  (gimp-image-undo-group-start img)

  (gimp-context-set-foreground color)
  (gimp-context-set-brush (car brush))
  (gimp-context-set-opacity (car (cdr brush)))(tracing 0)
  (gimp-context-set-paint-mode (car (cdr (cdr (cdr brush)))))

  (if (= drawab 1)
      (begin
        (aset *linepoints* 0 ax)
        (aset *linepoints* 1 ay)
        (aset *linepoints* 2 bx)
        (aset *linepoints* 3 by)
        (gimp-pencil drw 4 *linepoints*)
        )
  )

  (if (= drawcd 1) 
      (begin
      (aset *linepoints* 0 cx)
      (aset *linepoints* 1 cy)
      (aset *linepoints* 2 dx)
      (aset *linepoints* 3 dy)
      (gimp-pencil drw 4 *linepoints*)

      ))

  (if (< lineset 2)      ;  AB to CD / AB to DV
      (begin
        (set! niter 0)
        (while (<= niter ndiv)
               (set! x1 (+ ax (* niter axinc)))
               (set! y1 (+ ay (* niter ayinc)))

               (if (= lineset 0)
                   (begin
                     (set! x2 (+ cx (* niter cxinc)))
                     (set! y2 (+ cy (* niter cyinc))) )
                   )

               (if (= lineset 1)
                   (begin
                     (set! x2 (- dx (* niter cxinc)))
                     (set! y2 (- dy (* niter cyinc))) )
                   )
               
               (aset *linepoints* 0 x1)
               (aset *linepoints* 1 y1)
               (aset *linepoints* 2 x2)
               (aset *linepoints* 3 y2)
               (gimp-pencil drw 4 *linepoints*)

               (set! niter (+ 1 niter) ))))

  
   (if (= lineset 2)      ;  A to CD
       (begin
         (aset *linepoints* 0 ax)
         (aset *linepoints* 1 ay)))

   (if (= lineset 3)      ;  B to CD
       (begin
         (aset *linepoints* 0 bx)
         (aset *linepoints* 1 by)))
   
   
   (if ( or (= lineset 2) (= lineset 3))
       (begin
         (set! niter 0)
         (while (<= niter ndiv)
                (set! x2 (+ cx (* niter cxinc)))
                (set! y2 (+ cy (* niter cyinc)))
                (aset *linepoints* 2 x2)
                (aset *linepoints* 3 y2)
                (gimp-pencil drw 4 *linepoints*)
                (set! niter (+ 1 niter) )
                )))

     (if (= lineset 4)      ;  AB to C
            (begin
        (aset *linepoints* 0 cx)
        (aset *linepoints* 1 cy)))

     (if (= lineset 5)      ;  AB to D
            (begin
        (aset *linepoints* 0 dx)
        (aset *linepoints* 1 dy)))
    
   (if ( or (= lineset 4) (= lineset 5))
       (begin
        (set! niter 0)
        (while (<= niter ndiv)
            (set! x2 (+ ax (* niter axinc)))
            (set! y2 (+ ay (* niter ayinc)))
            (aset *linepoints* 2 x2)
            (aset *linepoints* 3 y2)
            (gimp-pencil drw 4 *linepoints*)
            (set! niter (+ 1 niter) )
        )))

    (gimp-image-undo-group-end img)
    (gimp-displays-flush)

    (gimp-context-pop)
        
   ) 

)

(script-fu-register "script-fu-linesegments"
                    _"Line Segments"
                    _"Draws lines between points on two lines."
                    "Charles Cave <charlesweb@optusnet.com.au>"
                    "Charles Cave"
                    "February 2008"
                    "RGB*, INDEXED*, GRAY*"
                    SF-IMAGE       "Image"         0
                    SF-DRAWABLE    "Drawable"      0
                    SF-ADJUSTMENT _"Segment divisions" '(20 2 300 1 10 0 1)
                    SF-ADJUSTMENT _"A x-coordinate " '(100 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"A y-coordinate " '(50 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"B x-coordinate " '(50 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"B y-coordinate " '(300 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"C x-coordinate " '(250 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"C y-coordinate " '(50 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"D x-coordinate " '(300 0 1000 1 10 0 1)
                    SF-ADJUSTMENT _"D y-coordinate " '(300 0 1000 1 10 0 1)
                    SF-OPTION     _"Lines to Draw"  '(_"AB to CD"
                                  _"AB to DC"
                                  _"A to CD"
                                  _"B to CD"
                                  _"AB to C"
                                  _"AB to D"
                                  _"No Lines")
                    SF-TOGGLE     _"Draw line AB"      TRUE
                    SF-TOGGLE     _"Draw line CD"      TRUE
                    
                    SF-BRUSH      _"Brush"         '("Circle (01)" 100 1 0)
                    SF-COLOR      _"Color"          "blue"
                    )

(script-fu-menu-register "script-fu-linesegments"
                         "<Image>/Filters")
